<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * AAM Plus Package backend
 *
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_PlusPackage_Backend {

    /**
     * Instance of itself
     * 
     * @var AAM_PlusPackage_Backend 
     * 
     * @access private
     */
    private static $_instance = null;
    
    /**
     * Initialize the extension
     * 
     * @return void
     * 
     * @access protected
     */
    protected function __construct() {
        //UI filters
        add_filter('aam-post-type-ui-filter', array($this, 'renderTypeUI'));
        add_filter('aam-term-type-ui-filter', array($this, 'renderTermUI'));
        add_filter('aam-ui-content-filter', array($this, 'renderUIContent'), 10, 2);
        add_filter('aam-post-option-preview-filter', array($this, 'postOptionPreview'), 10, 2);
        add_filter('aam-term-override-status', array($this, 'termOverrideStatus'), 10, 3);
        add_filter('aam-type-override-status', array($this, 'typeOverrideStatus'), 10, 3);
        add_filter('aam-get-post-access-filter', array($this, 'getAccessSettings'), 10, 2);
        add_filter('aam-term-row-actions', array($this, 'getTermRowActions'), 10, 4);
        
        //add custom ajax handler
        add_filter('aam-ajax-filter', array($this, 'ajax'), 10, 3);
        
        //add another option to the AAM Utilities extension
        add_action('aam-settings-filter', array($this, 'settingsList'), 10, 2);
        
        //post option list
        add_filter(
                'aam-post-access-options-filter', 
                array($this, 'filterPostAccessOptions'), 10, 2
        );
        
        if (AAM::api()->getConfig('core.settings.backendAccessControl', true)) {
            //taxonomy/term control
            add_filter('tag_row_actions', array($this, 'tagRowActions'), 10, 2);
            add_action('admin_init', array($this, 'adminInit'));
        }
        
        add_action('restrict_manage_posts', array($this, 'renderCategories'));
    }

    /**
     * Undocumented function
     *
     * @param [type] $actions
     * @param [type] $subject
     * @param [type] $term
     * @param [type] $postType
     * @return void
     */
    public function getTermRowActions($actions, $subject, $term, $postType) {
        if (in_array($subject->getUID(), array('user', 'role'))) {
            if (is_taxonomy_hierarchical($term->taxonomy)) {
                $tax       = $subject->getObject('taxonomy', "{$postType}|{$term->taxonomy}");
                $actions[] = ($tax && intval($tax->get('default')) === $term->term_id ? ',pinned' : ',pin');
            } else {
                $actions[] = 'no-pin';
            }
        }

        return $actions;
    }
    
     /**
     * Custom ajax handler
     * 
     * @param mixed            $response
     * @param AAM_Core_Subject $subject
     * @param string           $action
     * 
     * @return string
     * 
     * @access public
     */
    public function ajax($response, $subject, $action) {
        if ($action === 'PlusPackage.setDefaultTerm') {
            $id   = explode('|', filter_input(INPUT_POST, 'id'));
            $term = get_term($id[0], $id[1]);
            
            if (!is_wp_error($term)) {
                $taxonomy = $subject->getObject('taxonomy', "{$id[2]}|{$id[1]}");
                $taxonomy->save('default', $id[0]);
            }
            
            $response = json_encode(array('status' => 'success'));
        }
        
        return $response;
    }
    
    /**
     * 
     * @param type $settings
     * @param AAM_Core_Object $object
     * @return type
     */
    public function getAccessSettings($settings, AAM_Core_Object $object) {
        if ($object->getSubject()->getUID() === 'user') {
            $this->adjustSetting($settings, 'backend.edit', $object);
            $this->adjustSetting($settings, 'backend.list', $object);
            $this->adjustSetting($settings, 'backend.delete', $object);
            $this->adjustSetting($settings, 'backend.publish', $object);
            $this->adjustSetting($settings, 'frontend.list', $object);
            $this->adjustSetting($settings, 'frontend.read', $object);
            $this->adjustSetting($settings, 'api.list', $object);
            $this->adjustSetting($settings, 'api.read', $object);
            $this->adjustSetting($settings, 'api.edit', $object);
            $this->adjustSetting($settings, 'api.delete', $object);
        }
        
        return $settings;
    }
    
    /**
     * 
     * @param int $settings
     * @param type $prop
     * @param type $object
     */
    protected function adjustSetting(&$settings, $prop, $object) {
        if (!isset($settings[$prop]) && !empty($settings["{$prop}_others"])) {
            if (intval($object->post_author) !== intval($object->getSubject()->ID)) {
                $settings[$prop] = 1;
            }
        }
    }
    
    /**
     * 
     * @param type $type
     */
    public function renderCategories($type) {
        $api = AAM::api();
        
        if ($type === 'page' && $api->getConfig('core.settings.pageCategory', false)) {
            $this->renderCategoryDropdown('page_category');
        } elseif ($type === 'attachment' && $api->getConfig('core.settings.mediaCategory', false)) {
            $this->renderCategoryDropdown('media_category');
        } 
    }
    
    /**
     * 
     * @param type $taxonomy
     */
    protected function renderCategoryDropdown($taxonomy) {
        $args = array(
            'show_option_all' => get_taxonomy($taxonomy)->labels->all_items,
            'hide_empty'      => 0,
            'hierarchical'    => 1,
            'show_count'      => 0,
            'orderby'         => 'name',
            'selected'        => filter_input(INPUT_GET, $taxonomy),
            'taxonomy'        => $taxonomy,
            'name'            => $taxonomy,
            'value_field'     => 'slug'
        );

        echo '<label class="screen-reader-text" for="cat">' . __( 'Filter by category' ) . '</label>';
        wp_dropdown_categories($args);
    }
    
    /**
     * 
     * @param type $status
     * @param type $id
     * @param type $subject
     * @return type
     */
    public function termOverrideStatus($status, $id, $subject) {
        return $subject->getObject('term', $id)->isOverwritten();
    }
    
    /**
     * 
     * @param type $status
     * @param type $id
     * @param type $subject
     * @return type
     */
    public function typeOverrideStatus($status, $id, $subject) {
        return $subject->getObject('type', $id)->isOverwritten();
    }
    
    /**
     * 
     * @param type $options
     * @param type $area
     * @return type
     */
    public function filterPostAccessOptions($options, $area) {
        $options = $this->insertOptionAfter('list', $options, 'list_others', array(
            'title'   => __('List To Others', AAM_KEY),
            'exclude' => array('visitor', 'user'),
            'descr'   => sprintf(__('Similar to the LIST option however the %s will be hidden for all except the author. For more information check %sHelp Section%s.', AAM_KEY), '%s', "<a href='https://aamplugin.com/reference/plugin#list-to-others' target='_blank'>", '</a>'),
        ));
        
        if (in_array($area, array('frontend', 'api'))) {
            $options = $this->insertOptionAfter('read', $options, 'read_others', array(
                'title'   => __('Read By Others', AAM_KEY),
                'exclude' => array('visitor', 'user'),
                'descr'   => sprintf(__('Similar to the READ option however only the author of the %s will be able to read it. For others access is denied and user will be redirected based on the Access Denied Redirect rule. For more information check %sHelp Section%s.', AAM_KEY), '%s', "<a href='https://aamplugin.com/reference/plugin#read-by-others' target='_blank'>", '</a>')
            ));
        }  
        
        if (in_array($area, array('backend', 'api'))){
            $options = $this->insertOptionAfter('edit', $options, 'edit_others', array(
                'title'   => __('Edit By Others', AAM_KEY),
                'exclude' => array('visitor', 'user'),
                'descr'   => sprintf(__('Similar to the EDIT option however only the author of the %s will be able to edit it. For others access is denied and user will be redirected based on the Access Denied Redirect rule. For more information check %sHelp Section%s.', AAM_KEY), '%s', "<a href='https://aamplugin.com/reference/plugin#edit-by-others' target='_blank'>", '</a>'),
            ));
            $options = $this->insertOptionAfter('delete', $options, 'delete_others', array(
                'title'   => __('Delete By Others', AAM_KEY),
                'exclude' => array('visitor', 'user'),
                'descr'   => sprintf(__('Similar to the DELETE option however only the author of the %s will be able to trash or permanently delete it. For more information check %sHelp Section%s.', AAM_KEY), '%s', "<a href='https://aamplugin.com/reference/plugin#delete-by-others' target='_blank'>", '</a>'),
            ));
        }
        
        if ($area == 'backend') {
            $options = $this->insertOptionAfter('publish', $options, 'publish_others', array(
                'title' => __('Publish By Others', AAM_KEY),
                'exclude' => array('visitor', 'user'),
                'descr' => sprintf(__('Similar to the PUBLISH option however only the author of the %s will be able to publish it. For more information check %sHelp Section%s.', AAM_KEY), '%s', "<a href='https://aamplugin.com/reference/plugin#publish-by-others' target='_blank'>", '</a>'),
            ));
        }
        
        return $options;
    }
    
    /**
     * 
     * @param type $after
     * @param type $options
     * @param type $option
     * @param type $config
     * @return type
     */
    protected function insertOptionAfter($after, $options, $option, $config) {
        $updated = array();
        foreach($options as $key => $value) {
            $updated[$key] = $value;
            
            if ($key == $after) {
                $updated[$option] = $config;
            }
        }
        
        return $updated;
    }
    
    /**
     * Render post type UI
     * 
     * @return string
     * 
     * @access public
     */
    public function renderTypeUI() {
        return $this->preparePartial('post-type.phtml');
    }
    
    /**
     * 
     * @param type $value
     * @param type $option
     * @return type
     */
    public function postOptionPreview($value, $option) {
        if (!empty($value) && $option == 'post|frontend.location') {
            $chunks = explode('|', $value);
            
            if ($chunks[0] == 'page') {
                $value = __('Existing Page', AAM_KEY);
            } elseif ($chunks[0] == 'url') {
                $value = __('Valid URL', AAM_KEY);
            } elseif ($chunks[0] == 'callback') {
                $value = __('Custom Callback', AAM_KEY);
            }
        } elseif (!empty($value) && $option == 'post|frontend.teaser') {
            $value = wp_strip_all_tags($value);
            if (function_exists('mb_strlen')) {
                $value = (mb_strlen($value) > 25 ? mb_substr($value, 0, 22) . '...' : $value);
            } else {
                $value = (strlen($value) > 25 ? substr($value, 0, 22) . '...' : $value);
            }
        }
        
        return $value;
    }
    
    /**
     * 
     * @staticvar type $list
     * @param type $area
     * @param type $section
     * @return type
     */
    public function getTypeOptionList($area, $section) {
        static $list = null;
        
        if (is_null($list)) {
            $list = require_once dirname(__FILE__) . '/Options/Type.php';
        }
        
        $options = apply_filters(
                'aam-type-access-options-filter', 
                $list[$area][$section], 
                $area, 
                $section
        );
        
        return $this->filterOptionList($options);
    }
    
    /**
     * 
     * @staticvar type $list
     * @param type $area
     * @param type $section
     * @return type
     */
    public function getTermOptionList($area, $section) {
        static $list = null;
        
        if (is_null($list)) {
            $list = require_once dirname(__FILE__) . '/Options/Term.php';
        }
        
        $options = apply_filters(
                'aam-term-access-options-filter', 
                $list[$area][$section], 
                $area, 
                $section
        );
        
        return $this->filterOptionList($options);
    }
    
    /**
     * 
     * @param type $options
     * @return type
     */
    protected function filterOptionList($options) {
        $subject = AAM_Backend_Subject::getInstance()->getUID();
        
        $filtered = array();
        foreach($options as $option => $data) {
            $add = empty($data['exclude']) || !in_array($subject, $data['exclude']);
            
            if ($add) {
                $add = empty($data['config']) || AAM::api()->getConfig(
                       $data['config'], true
                ); 
            }
            
            if ($add) {
                $filtered[$option] = $data;
            }
        }
        
        return $filtered;
    }
    
    /**
     * 
     * @param type $list
     * @param type $group
     * @return type
     */
    public function settingsList($list, $group) {
        if ($group == 'post') {
            $list['core.settings.pageCategory'] = array(
                'title'    => __('Page Categories', AAM_KEY),
                'descr'    => __('Allow to group pages into categories the same way as it is done with posts. This is a good way to group multiple pages in categories to simplify access management.', AAM_KEY),
                'value'    => AAM::api()->getConfig('core.settings.pageCategory', false),
                'category' => 'post'
            );
            $list['core.settings.mediaCategory'] = array(
                'title'    => __('Media Categories', AAM_KEY),
                'descr'    => __('Allow to group media into categories the same way as it is done with posts. This is a good way to group multiple pages in categories to simplify access management.', AAM_KEY),
                'value'    => AAM::api()->getConfig('core.settings.mediaCategory', false),
                'category' => 'post'
            );
            $list['core.settings.multiCategory'] = array(
                'title'    => __('Support Multiple Categories', AAM_KEY),
                'descr'    => AAM_Backend_View_Helper::preparePhrase('[Note!] This feature may slowdown your website execution. When more than one category or custom taxonomy is assigned to a post, AAM takes in consideration only the first one. If this option is checked, AAM will merge access settings from multiple categories.', 'b'),
                'value'    => AAM::api()->getConfig('core.settings.multiCategory', false),
                'category' => 'post'
            );
            $list['core.settings.inheritParentPost'] = array(
                'title'    => __('Inherit From Parent Post', AAM_KEY),
                'descr'    => AAM_Backend_View_Helper::preparePhrase('[Use cautiously!] Allow to inheritance access settings from a parent post. If post type support hierarchical structure (for example pages), then AAM will inherit settings from a parent post when present.', 'b'),
                'value'    => AAM::api()->getConfig('core.settings.inheritParentPost', false),
                'category' => 'post'
            );
        }
         
        return $list;
    }
    
    /**
     * 
     * @param type $content
     * @param type $view
     * @return type
     */
    public function renderUIContent($content, $view) {
        if ($view == 'postform') {
            $otype = filter_input(INPUT_POST, 'otype');
            if ($otype == 'term') {
                $content = $this->renderTermUI();
            }
        }
        
        return $content;
    }

    /**
     * Render term type UI
     * 
     * @return string
     * 
     * @access public
     */
    public function renderTermUI() {
        return $this->preparePartial('term-type.phtml');
    }
    
    /**
     * Render term type UI
     * 
     * @return string
     * 
     * @access public
     */
    protected function preparePartial($tmpl) {
        ob_start();
        require dirname(__FILE__) . '/view/' . $tmpl;
        $content = ob_get_contents();
        ob_end_clean();

        return $content;
    }
    
    /**
     * Check if term need to be filtered
     * 
     * @param string $id
     * @param string $taxonomy
     * @param string $action
     * 
     * @return boolean
     * 
     * @access protected
     */
    protected function checkTerm($id, $taxonomy, $action) {
        $response = false;
        
        $object = AAM_PlusPackage_Base::getInstance()->initTermAccessSettings(
                $id, $taxonomy
        );
        
        if ($object->has('term', $action)) {
            $response = true;
        }

        return $response;
    }

    /**
     * Term Quick Menu Actions Filtering
     *
     * @param array $actions
     * @param object $term
     *
     * @return array
     *
     * @access public
     */
    public function tagRowActions($actions, $term) {
        //filter edit menu
        if ($this->checkTerm($term->term_id, $term->taxonomy, 'backend.edit')) {
            if (isset($actions['edit'])) {
                unset($actions['edit']);
            }
            if (isset($actions['inline hide-if-no-js'])) {
                unset($actions['inline hide-if-no-js']);
            }
        }

        //filter delete menu
        if ($this->checkTerm($term->term_id, $term->taxonomy, 'backend.delete')) {
            if (isset($actions['delete'])) {
                unset($actions['delete']);
            }
        }
        
        return $actions;
    }
    
    /**
     * Control Edit Term
     *
     * Make sure that current user does not have access to edit Term
     *
     * @return void
     *
     * @access public
     */
    public function adminInit() {
        $tagId    = isset($_REQUEST['tag_ID']) ? $_REQUEST['tag_ID'] : null;
        $taxonomy = isset($_REQUEST['taxonomy']) ? $_REQUEST['taxonomy'] : null;
            
        if ($taxonomy && $tagId) {
            if ($this->checkTerm($tagId, $taxonomy, 'backend.edit')) {
                AAM_Core_API::reject('backend', 
                    array(
                        'hook'     => 'term_edit', 
                        'action'   => 'backend.edit', 
                        'term'     => $tagId,
                        'taxonomy' => $taxonomy
                    )
                );
            }
        }
    }

    /**
     * Bootstrap the backend
     * 
     * @return AAM_PlusPackage_Backend
     * 
     * @access public
     */
    public static function bootstrap() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self;
        }

        return self::$_instance;
    }

}