<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * AAM Plus Package
 *
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_PlusPackage {

    /**
     * Instance of itself
     * 
     * @var AAM_PlusPackage 
     * 
     * @access private
     * @static
     */
    private static $_instance = null;
    
    /**
     * Initialize the extension
     * 
     * @return void
     * 
     * @access protected
     */
    protected function __construct() {
        //post access filter
        add_filter('aam-post-access-filter', array($this, 'getPostAccess'), 1, 2);
        
        //term access filter
        add_filter('aam-term-access-filter', array($this, 'getTermAccess'), 1, 2);
        
        //type access filter
        add_filter('aam-type-access-filter', array($this, 'getTypeAccess'), 1, 2);
        
        //register Type & Term custom objects
        add_filter('aam-object-filter', array($this, 'objectFilter'), 10, 4);
        
        //URI access control
        add_filter('aam-uri-match-filter', array($this, 'matchURI'), 10, 3);
        
        //register taxonomies
        add_action('init', array($this, 'init'), 1);
        
        if (is_admin()) {
            AAM_PlusPackage_Backend::bootstrap();
        } elseif (AAM::api()->getConfig('core.settings.frontendAccessControl', true)) {
            AAM_PlusPackage_Frontend::bootstrap();
        }
        
        if (AAM::api()->getConfig('core.settings.apiAccessControl', true)) {
            add_filter(
                'rest_request_before_callbacks', 
                'AAM_PlusPackage_Api_Manager::dispatch', 
                10, 
                3
            );
        }
        
        // Register joined functionality for all areas
        AAM_PlusPackage_Base::bootstrap();
    }
    
    /**
     * 
     */
    public function init() {
        $this->registerPageCategory();
        $this->registerMediaCategory();
    }
    
    /**
     * 
     * @param type $matched
     * @param type $uri
     * @param type $s
     * @return type
     */
    public function matchURI($matched, $uri, $s) {
        $regex = '@^' . str_replace('\*', '(.*)', preg_quote($uri['path'])) . '$@';
        
        return preg_match($regex, $s);
    }
    
    /**
     * Register page category taxonomy
     * 
     * @access protected
     * @return void
     */
    protected function registerPageCategory() {
        if (AAM::api()->getConfig('core.settings.pageCategory', false)) {
            register_taxonomy('page_category', 'page', array(
                'hierarchical'      => true,
                'rewrite'           => true,
                'public'            => true,
                'show_ui'           => true,
                'show_in_nav_menus' => true,
            ));
        }
    }
    
    /**
     * Register media category taxonomy
     * 
     * @access protected
     * @return void
     */
    protected function registerMediaCategory() {
        if (AAM::api()->getConfig('core.settings.mediaCategory', false)) {
            register_taxonomy('media_category', 'attachment', array(
                'hierarchical'      => true,
                'rewrite'           => true,
                'public'            => true,
                'show_ui'           => true,
                'show_in_nav_menus' => true,
            ));
        }
    }

    /**
     * Get post access
     * 
     * @param array|null           $options
     * @param AAM_Core_Object_Post $post
     * 
     * @return array|null
     * 
     * @access public
     */
    public function getPostAccess($options, AAM_Core_Object_Post $post) {
        $allow = AAM::api()->getConfig('core.settings.inheritParentPost', false);
        
        if (empty($options) && $allow && $post->post_parent) {
            $parent  = $post->getSubject()->getObject(
                'post', $post->post_parent, array('voidInheritance' => true)
            );
            $options = $parent->getOption();
        }
        
        if (empty($options)) {
            $options = $this->inheritFromPostParent($post);
        }
        
        return $options;
    }
    
    /**
     * Inherit post access from parent object
     * 
     * If post by itself does not have any settings, the system tries to inherit
     * settings from the parent Term and Post Type.
     * 
     * @return array|null
     * 
     * @access protected
     */
    protected function inheritFromPostParent(AAM_Core_Object_Post $post) {
        //try to inherit it from parent term if exists
        $terms = AAM_PlusPackage_Base::getInstance()->retrievePostTerms($post->getPost());
        
        if (!is_wp_error($terms) && count($terms)) {
            if (AAM::api()->getConfig('core.settings.multiCategory', false)) {
                $collection = array();
                foreach($terms as $term) {
                    $collection[] = $this->inheritFromTerm($term, $post->getSubject());
                }
                $option = $this->mergeSettings($collection);
            } else {
                $option = $this->inheritFromTerm(array_shift($terms), $post->getSubject());
            }
        }
        
        //if empty, inherit from the default post type settings
        if (empty($option)) {
            $object = new AAM_Object_Type(
                    $post->getSubject(), $post->getPost()->post_type
            );
            $option = $object->getGroup('post');
        }

        return $option;
    }
    
    /**
     * Merge terms access settings
     * 
     * @param array $collection
     * 
     * @access protected
     * @return array
     */
    protected function mergeSettings($collection) {
        $merged = array();
        $access = AAM::api()->getConfig('access.merging.preference', 'deny');
        
        // first get the complete list of unique keys
        $keys = array_keys(call_user_func_array('array_merge', $collection));
        
        foreach($keys as $key) {
            foreach($collection as $options) {
                // If merging preference is "deny" and at least one of the term
                // settings has it checked, then final merged array will have it set
                // to checked
                if ($access == 'deny' && !empty($options[$key])) {
                    $merged[$key] = $options[$key];
                    break;
                } elseif ($access == 'allow' && empty($options[$key])) {
                    $merged[$key] = 0;
                    break;
                } elseif (isset($options[$key])) {
                    $merged[$key] = $options[$key];
                }
            }
        }
        
        return $merged;
    }
    
    /**
     * Inherit access from parent term
     * 
     * Go through the hierarchical branch of terms and retrieve access from the 
     * first parent term that has access defined.
     * 
     * @param stdClass $term
     * 
     * @return array
     * 
     * @access private
     */
    private function inheritFromTerm($term, AAM_Core_Subject $subject) {
        $object = new AAM_Object_Term($subject, $term->term_id, $term->taxonomy);
        $option = $object->getGroup('post');
        
        return $option;
    }
    
    /**
     * Get term access
     * 
     * @param array|null      $options
     * @param AAM_Object_Term $object
     * 
     * @return array|null
     * 
     * @access public
     */
    public function getTermAccess($options, AAM_Object_Term $object) {
        $subject = $object->getSubject();
        $term    = $object->getTerm();

        if (empty($options)) { //inherit from default
            $taxonomy = get_taxonomy($term->taxonomy);
            if (isset($taxonomy->object_type[0])) {
                $postType = $taxonomy->object_type[0];
                $type     = new AAM_Object_Type($subject, $postType);
                $options  = $type->getOption();
            }
        }

        if (empty($options)) { //inherit from parent subject
            $options = $subject->inheritFromParent(
                'term', "{$term->term_id}|{$term->taxonomy}"
            );
        }
        
        // TODO: Figure out why $postType can be empty with polylang plugin active
        if (empty($options) && !empty($postType)) { //inherit from parent default settings
            $options = $subject->inheritFromParent('type', $postType);
        }
        
        return $options;
    }
    
    /**
     * Get default post type access
     * 
     * @param array|null      $options
     * @param AAM_Object_Term $object
     * 
     * @return array|null
     * 
     * @access public
     */
    public function getTypeAccess($options, AAM_Object_Type $object) {
        $subject = $object->getSubject();

        if (empty($options)) { //inherit from parent default settings
            $options = $subject->inheritFromParent('type', $object->getPostType());
        }
        
        return $options;
    }
    
    /**
     * Get default post type access
     * 
     * @param array|null      $options
     * @param AAM_Object_Term $object
     * 
     * @return array|null
     * 
     * @access public
     */
    public function getTaxonomyAccess($options, AAM_Object_Taxonomy $object) {
        $subject = $object->getSubject();

        if (empty($options)) { //inherit from parent default settings
            $options = $subject->inheritFromParent('taxonomy', $object->getId());
        }
        
        if (empty($options) && $subject->isDefault()) {
            $default = get_option('default_' . $object->getTaxonomy());
            if (!empty($default)) {
                $options = array('default' => $default);
            }
        }
        
        return $options;
    }

    /**
     * AAM object filter
     * 
     * @param mixed            $object
     * @param string           $objectType
     * @param string|int       $id
     * @param AAM_Core_Subject $subject
     * 
     * @return AAM_Object_Term|AAM_Object_Type
     * 
     * @access public
     */
    public function objectFilter($object, $objectType, $id, $subject) {
        if ($objectType === 'term') {
            list($termId, $taxonomy) = explode('|', $id);
            $object  = new AAM_Object_Term($subject, $termId, $taxonomy);
            $options = $object->getOption();
            
            if (empty($options)) {
                $object->setOption($this->getTermAccess($options, $object));
            }
        } elseif ($objectType === 'type') {
            $object  = new AAM_Object_Type($subject, $id);
            $options = $object->getOption();
            
            if (empty($options)) {
                $object->setOption($this->getTypeAccess($options, $object));
            }
        } elseif ($objectType === 'taxonomy') {
            $object  = new AAM_Object_Taxonomy($subject, $id);
            $options = $object->getOption();
            
            if (empty($options)) {
                $object->setOption($this->getTaxonomyAccess($options, $object));
            }
        }
        
        return $object;
    }
    
    /**
     * Bootstrap the extension
     * 
     * @return AAM_PlusPackage
     * 
     * @access public
     * @static
     */
    public static function bootstrap() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self;
        }

        return self::$_instance;
    }

}