<?php

/**
 * Copyright (C) VasylTech, LLC <vasyl@vasyltech.com>
 *
 * This program is commercial software: you are not allowed to redistribute it
 * and/or modify. Unauthorized copying of this file, via any medium is strictly
 * prohibited.
 * For any questions or concerns contact Vasyl Martyniuk <vasyl@vasyltech.com>
 */

/**
 * AAM Plus Package Visibility Query builder
 *
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 * @since v3.9
 */
class AAM_PlusPackage_Visibility_QueryBuilder {

    /**
     *
     * @var AAM_PlusPackage_Visibility_QueryBuilder 
     */
    protected static $instance = null;

    /**
     * 
     */
    protected function __construct() {}

    /**
     * 
     * @global type $wpdb
     * @param string $where
     * @param type $wpQuery
     * @param type $option
     * @return string
     */
    public function execute($where, $wpQuery, $option) {
        $groups = array();
        $area   = $this->getArea();
        $userId = intval(get_current_user_id());
        
        foreach($this->getPostType($wpQuery) as $postType) {
            $args = $this->prepareQueryArgs($option, $postType, $area);
            
            if ($args['is_default_list'] === true) {
                $query = $this->buildListBasedQuery($args, $postType, $userId);
            } elseif ($args['is_default_author'] === true) {
                $query = $this->buildAuthorBasedQuery($args, $postType, $userId);
            } else {
                $query = $this->buildQuery($args, $postType, $userId);
            }
            
            if (!empty($query)) {
                $groups[] = $query;
            }
        }
        
        if (!empty($groups)) {
            $where = ' AND ((' . implode(') OR (', $groups) . '))';
        }
        
        return $where;
    }
    
    /**
     * 
     * @global type $wpdb
     * @param type $args
     * @param type $postType
     * @param type $userId
     * @return string
     */
    protected function buildListBasedQuery($args, $postType, $userId) {
        global $wpdb;
        
        $query = '('; // Start the group segment;
        
        $query .= "({$wpdb->posts}.post_type = '{$postType}' ";
        $query .= "AND {$wpdb->posts}.ID = 0)";
        
        // == Building query based on LIST args ==
        $in = array();

        // If there is any term that allows LIST, then exclude it from filtering
        if (!empty($args['term__in'])) {
            $field = "{$wpdb->term_relationships}.term_taxonomy_id";
            $in[]  = "{$field} IN (" . implode(',', $args['term__in']) . ')';
        }
        
        // If there is any post that allows LIST, then exclude it from filtering
        if (!empty($args['post__in'])) {
            $in[] = "{$wpdb->posts}.ID IN (" . implode(',', $args['post__in']) . ")";
        }

        if (!empty($in)) {
            $query .= ' OR (' . implode(' OR ', $in) . ')';
        }
        
        // == Building query based on LIST_TO_OTHERS args ==
        $author = array();

        // If there is any term that exlicitly states that to show all posts that 
        // belong to an author, then exclude them from filtering
        if (!empty($args['term_author__in'])) {
            $ids      = implode(',', $args['term_author__in']);
            $author[] = "{$wpdb->term_relationships}.term_taxonomy_id IN ({$ids})";
        }
        
        // If there is any post that explicitly should be shown for author, then exclude
        // it from the filtering
        if (!empty($args['post_author__in'])) {
            $ids      = implode(',', $args['post_author__in']);
            $author[] = "{$wpdb->posts}.ID IN ({$ids})";
        }
        
        if (!empty($author)) {
            $query .= ' OR ((' . implode(' OR ', $author) . ") ";
            $query .= "AND ({$wpdb->posts}.post_author = {$userId}))";
        }
        
        $query .= ')'; // End the group segment
        
        // However if there is any post that explicitly has LIST checked, then 
        // filter it out
        if (!empty($args['post__not_in'])) {
            $postIds = implode(',', $args['post__not_in']);
            $query .= " AND {$wpdb->posts}.ID NOT IN ({$postIds})";
        }
        
        return $query;
    }
    
    /**
     * 
     * @global type $wpdb
     * @param type $args
     * @param type $postType
     * @param type $userId
     * @return type
     */
    protected function buildAuthorBasedQuery($args, $postType, $userId) {
        global $wpdb;
        
        $query = '('; // Start the group segment;
        
        $query .= "({$wpdb->posts}.post_type = '{$postType}' ";
        $query .= "AND {$wpdb->posts}.post_author = {$userId})";
        
        $author = array();

        // If there are any terms that have explicitly unchecked LIST TO OTHERS, then
        // make sure that those terms are not filtered
        if (!empty($args['term_author__not_in'])) {
            $ids      = implode(',', $args['term_author__not_in']);
            $author[] = "{$wpdb->term_relationships}.term_taxonomy_id IN ({$ids})";
        }
        
        // If there are any posts that have explicitly unchecked LIST TO OTHERS or LIST,
        // then make sure that those posts are not filtered
        $exclude = array_merge(
            (!empty($args['post_author__not_in']) ? $args['post_author__not_in'] : array()),
            (!empty($args['post__in']) ? $args['post__in'] : array())
        );
        if (!empty($exclude)) {
            $ids      = implode(',', $exclude);
            $author[] = "{$wpdb->posts}.ID IN ({$ids})";
        }

        if (!empty($author)) {
            $query .= ' OR (' . implode(' OR ', $author) . ')';
        }
        
        $query .= ')'; // End the group segment
        
        // However if there is any post or term that explicitly has LIST checked, 
        // then filter it out
        if (!empty($args['post__not_in'])) {
            $ids    = implode(',', $args['post__not_in']);
            $query .= " AND {$wpdb->posts}.ID NOT IN ({$ids})";
        }
        if (!empty($args['term__not_in'])) {
            $ids    = implode(',', $args['term__not_in']);
            $query .= " AND ({$wpdb->posts}.ID NOT IN (SELECT object_id FROM {$wpdb->term_relationships} WHERE term_taxonomy_id IN ({$ids})))";
        }
        
        return $query;
    }
    
    
    /**
     * 
     * @param type $args
     * @param type $userId
     * 
     * @return string
     */
    protected function buildQuery($args, $postType, $userId) {
        global $wpdb;
        
        $query = ''; // Start the group segment;
        
        $parts = array();

        // First filter out all terms that have explicitly checked LIST option
        if (!empty($args['term__not_in'])) {
            $ids   = implode(',', $args['term__not_in']);
            $part  = "({$wpdb->posts}.ID NOT IN (SELECT object_id FROM {$wpdb->term_relationships} WHERE term_taxonomy_id IN ({$ids})))";
            
            $parts[] = $part;
        }
        
        // Now filter out all terms that have explicitly checked LIST TO OTHERS
        if (!empty($args['term_author__in'])) {
            $ids   = implode(',', $args['term_author__in']);
            
            $part  = "(({$wpdb->posts}.ID NOT IN (SELECT object_id FROM {$wpdb->term_relationships} WHERE term_taxonomy_id IN ({$ids}))) ";
            $part .= "OR ({$wpdb->posts}.post_author = {$userId}))";
            
            $parts[] = $part;
        }
        
        // Filter out all posts that have explicitly checked LIST
        if (!empty($args['post__not_in'])) {
            $ids     = implode(',', $args['post__not_in']);
            $parts[] = "{$wpdb->posts}.ID NOT IN ({$ids})";
        }
        
        // Now filter out all posts that have explicitly checked LIST TO OTHERS
        if (!empty($args['post_author__in'])) {
            $ids   = implode(',', $args['post_author__in']);
            
            $part  = "({$wpdb->posts}.ID NOT IN ({$ids}) ";
            $part .= "OR ({$wpdb->posts}.post_author = {$userId}))";
            
            $parts[] = $part;
        }
        
        if (count($parts)) {
            $query .= '(' . implode(' AND ', $parts) . ')';
        }
        
        // Finally include all posts that explicitly have unchecked LIST or
        // LIST TO OTHERS BUT ONLY if there are any filters defined
        if (!empty($query) 
                && (!empty($args['post__in']) || !empty($args['post_author__not_in']))) {
            $postIn   = (isset($args['post__in']) ? $args['post__in'] : array());
            $authorIn = (isset($args['post_author__not_in']) ? $args['post_author__not_in'] : array());
            
            $ids = implode(',', array_merge($postIn, $authorIn));
            $query .= " OR {$wpdb->posts}.ID IN ({$ids})";
        }
        
        return "{$wpdb->posts}.post_type = '{$postType}'" . (!empty($query) ? " AND ({$query})" : '');
    }
    
    /**
     * 
     * @param type $visibilities
     * @param type $postType
     * @param type $area
     * @return type
     */
    protected function prepareQueryArgs($visibilities, $postType, $area) {
        $args = array(
            'is_default_list'   => false,
            'is_default_author' => false
        );
        
        if (isset($visibilities['type'][$postType])) {
            $ops = $visibilities['type'][$postType];

            if (!empty($ops["post|{$area}.list"])) {
                $args['is_default_list'] = true;
            } elseif (!empty($ops["post|{$area}.list_others"])) {
                $args['is_default_author'] = true;
            }
        }
        
        if (isset($visibilities['term'])) {
            $args = array_merge(
                $args, 
                $this->prepareTermQueryArgs($visibilities['term'], $postType, $area)
            );
        }
        
        if (!empty($visibilities['post'])) {
            $args = array_merge(
                $args, 
                $this->preparePostQueryArgs($visibilities['post'], $postType, $area)
            );
        }
        
        return $args;
    }
    
    /**
     * 
     * @param type $terms
     * @param type $postType
     * @param type $area
     * @return type
     */
    protected function prepareTermQueryArgs($terms, $postType, $area) {
        $taxonomies = get_object_taxonomies($postType);
        
        $response = array(
            'term__in'            => array(),
            'term__not_in'        => array(),
            'term_author__in'     => array(),
            'term_author__not_in' => array()
        );

        foreach ($terms as $id => $ops) {
            $chunks = explode('|', $id);
            if (in_array($chunks[1], $taxonomies)) {
                if (isset($ops["post|{$area}.list"])) {
                    if (!empty($ops["post|{$area}.list"])) {
                        $response['term__not_in'][] = $chunks[0];
                    // NOTE! Covering scenario when LIST is unchecked and LIST TO OTHERS
                    // is checked
                    } elseif (empty($ops["post|{$area}.list_others"])) {
                        $response['term__in'][] = $chunks[0];
                    }
                } 
                
                // LIST has higher priority than LIST TO OTHERS
                if (empty($ops["post|{$area}.list"]) && isset($ops["post|{$area}.list_others"])) {
                    // Note! If LIST_TO_OTHERS is checked, then filter out term from
                    // all users that NOT authors of child posts that is why the
                    // "not_in" is populated in reverse order than LIST
                    if (!empty($ops["post|{$area}.list_others"])) {
                        $response['term_author__in'][] = $chunks[0];
                    } elseif(!isset($ops["post|{$area}.list"])) {
                        $response['term_author__not_in'][] = $chunks[0];
                    }
                }
            }
        }
        
        return $response;
    }
    
    /**
     * 
     * @param type $posts
     * @param type $postType
     * @param type $area
     * @return type
     */
    protected function preparePostQueryArgs($posts, $postType, $area) {
        $response = array(
            'post__in'            => array(),
            'post__not_in'        => array(),
            'post_author__in'     => array(),
            'post_author__not_in' => array()
        );
        
        foreach ($posts as $id => $ops) {
            $chunks = explode('|', $id);

            if ($postType == $chunks[1]) {
                if (isset($ops["{$area}.list"])) {
                    if (!empty($ops["{$area}.list"])) {
                        $response['post__not_in'][] = $chunks[0];
                    // NOTE! Covering scenario when LIST is unchecked and LIST TO OTHERS
                    // is checked
                    } else {
                        $response['post__in'][] = $chunks[0];
                    }
                } 
                
                // LIST has higher priority that LIST TO OTHERS
                if (empty($ops["{$area}.list"]) && isset($ops["{$area}.list_others"])) {
                    // Note! If LIST_TO_OTHERS is checked, then filter out post from
                    // all users that NOT authors of a post that is why the not_in
                    // is populated in reverse order than LIST
                    if (!empty($ops["{$area}.list_others"])) {
                        $response['post_author__in'][] = $chunks[0];
                    } elseif(!isset($ops["{$area}.list"])) {
                        $response['post_author__not_in'][] = $chunks[0];
                    }
                }
            }
        }
        
        return $response;
    }
    
    /**
     * 
     * @param type $query
     * 
     * @return array
     */
    protected function getPostType($query) {
        $postType = 'any';
        
        if (!empty($query->query['post_type'])) {
            $postType = $query->query['post_type'];
        } elseif (!empty($query->query_vars['post_type'])) {
            $postType = $query->query_vars['post_type'];
        } elseif ($query->is_attachment) {
            $postType = 'attachment';
        } elseif ($query->is_page) {
            $postType = 'page';
        } elseif ($query->is_tax) {
            $postType = array();
            $taxonomies = array_keys($query->tax_query->queried_terms);
            
            foreach (get_post_types(array('exclude_from_search' => false)) as $pt) {
                if ($pt === 'attachment') {
                    $object_taxonomies = get_taxonomies_for_attachments();
                } else {
                    $object_taxonomies = get_object_taxonomies($pt);
                }
                
                if (array_intersect($taxonomies, $object_taxonomies)) {
                    $postType[] = $pt;
                }
            }
        }
        
        if ($postType == 'any') {
            $postType = array_keys(
                get_post_types(
                    array('public' => true, 'exclude_from_search' => false), 
                    'names'
                )
            );
        }
        
        return (array) $postType;
    }
    
    /**
     * 
     * @return type
     */
    protected function getArea() {
        if (defined('REST_REQUEST') && REST_REQUEST) {
            $area = 'api';
        } else {
            $area = (is_admin() ? 'backend' : 'frontend');
        }
        
        return $area;
    }

    /**
     * 
     */
    public static function bootstrap() {
        if (is_null(self::$instance)) {
            self::$instance = new self;
        }
        
        return self::$instance;
    }

}